@@ -20,7 +20,7 @@ import subprocess
 import time
 import tempfile
 
-os.environ["PATH"] = "/home/schanzen/gnunet/build/bin" + ":" + os.environ["PATH"]
+os.environ["PATH"] = "/home/grothoff/bin" + ":" + os.environ["PATH"]
 
 if os.name == "nt":
     tmp = os.getenv("TEMP")
