@@ -172,7 +172,7 @@ modify_request (void *cls,
                "Injecting modified DNS response\n");
     GNUNET_DNS_request_answer (rh, len, buf);
   }
-  GNUNET_free_non_null (buf);
+  GNUNET_free (buf);
 }
 
 
@@ -260,7 +260,7 @@ main (int argc, char *const *argv)
                                ("Change DNS replies to point elsewhere."),
                              options,
                              &run, NULL)) ? ret : 1;
-  GNUNET_free ((void *) argv);
+  GNUNET_free_nz ((void *) argv);
   return ret;
 }
 
