@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     Copyright (C) 2006-2020 GNUnet e.V.
+     Copyright (C) 2006-2021 GNUnet e.V.
 
      GNUnet is free software: you can redistribute it and/or modify it
      under the terms of the GNU Affero General Public License as published
@@ -143,36 +143,55 @@ enum GNUNET_GenericReturnValue
  */
 
 #if __BYTE_ORDER == __LITTLE_ENDIAN
-#define GNUNET_htobe16(x) __bswap_16 (x)
+#if defined(__linux__)
+#define BYTE_SWAP_16(x) __bswap_16 (x)
+#define BYTE_SWAP_32(x) __bswap_32 (x)
+#define BYTE_SWAP_64(x) __bswap_64 (x)
+#else
+#define BYTE_SWAP_16(x) ((((x) & 0x00ff) << 8) | (((x) & 0xff00) >> 8))
+
+#define BYTE_SWAP_32(x)                                   \
+  ((((x) & 0x000000ffU) << 24) | (((x) & 0x0000ff00U) << 8)   \
+   | (((x) & 0x00ff0000U) >> 8) | (((x) & 0xff000000U) >> 24))
+
+#define BYTE_SWAP_64(x)                                                      \
+  ((((x) & 0x00000000000000ffUL) << 56) | (((x) & 0x000000000000ff00UL) << 40)   \
+   | (((x) & 0x0000000000ff0000UL) << 24) | (((x) & 0x00000000ff000000UL) << 8)    \
+   | (((x) & 0x000000ff00000000UL) >> 8) | (((x) & 0x0000ff0000000000UL) >> 24)    \
+   | (((x) & 0x00ff000000000000UL) >> 40) | (((x) & 0xff00000000000000UL) >> \
+                                             56))
+#endif
+
+#define GNUNET_htobe16(x) BYTE_SWAP_16 (x)
 #define GNUNET_htole16(x) (x)
-#define GNUNET_be16toh(x) __bswap_16 (x)
+#define GNUNET_be16toh(x) BYTE_SWAP_16 (x)
 #define GNUNET_le16toh(x) (x)
 
-#define GNUNET_htobe32(x) __bswap_32 (x)
+#define GNUNET_htobe32(x) BYTE_SWAP_32 (x)
 #define GNUNET_htole32(x) (x)
-#define GNUNET_be32toh(x) __bswap_32 (x)
+#define GNUNET_be32toh(x) BYTE_SWAP_32 (x)
 #define GNUNET_le32toh(x) (x)
 
-#define GNUNET_htobe64(x) __bswap_64 (x)
+#define GNUNET_htobe64(x) BYTE_SWAP_64 (x)
 #define GNUNET_htole64(x) (x)
-#define GNUNET_be64toh(x) __bswap_64 (x)
+#define GNUNET_be64toh(x) BYTE_SWAP_64 (x)
 #define GNUNET_le64toh(x) (x)
 #endif
 #if __BYTE_ORDER == __BIG_ENDIAN
 #define GNUNET_htobe16(x) (x)
-#define GNUNET_htole16(x) __bswap_16 (x)
+#define GNUNET_htole16(x) BYTE_SWAP_16 (x)
 #define GNUNET_be16toh(x) (x)
-#define GNUNET_le16toh(x) __bswap_16 (x)
+#define GNUNET_le16toh(x) BYTE_SWAP_16 (x)
 
 #define GNUNET_htobe32(x) (x)
-#define GNUNET_htole32(x) __bswap_32 (x)
+#define GNUNET_htole32(x) BYTE_SWAP_32 (x)
 #define GNUNET_be32toh(x) (x)
-#define GNUNET_le32toh(x) __bswap_32 (x)
+#define GNUNET_le32toh(x) BYTE_SWAP_32 (x)
 
 #define GNUNET_htobe64(x) (x)
-#define GNUNET_htole64(x) __bswap_64 (x)
+#define GNUNET_htole64(x) BYTE_SWAP_64 (x)
 #define GNUNET_be64toh(x) (x)
-#define GNUNET_le64toh(x) __bswap_64 (x)
+#define GNUNET_le64toh(x) BYTE_SWAP_64 (x)
 #endif
 
 
@@ -416,11 +435,12 @@ enum GNUNET_ErrorType
  * @param date when was the message logged?
  * @param message what is the message
  */
-typedef void (*GNUNET_Logger) (void *cls,
-                               enum GNUNET_ErrorType kind,
-                               const char *component,
-                               const char *date,
-                               const char *message);
+typedef void
+(*GNUNET_Logger) (void *cls,
+                  enum GNUNET_ErrorType kind,
+                  const char *component,
+                  const char *date,
+                  const char *message);
 
 
 /**
@@ -855,6 +875,24 @@ GNUNET_error_type_to_string (enum GNUNET_ErrorType kind);
  * @ingroup logging
  * Use this for fatal errors that cannot be handled
  */
+#if __GNUC__ >= 6  || __clang_major__ >= 6
+#define GNUNET_assert(cond)                                     \
+  do                                                            \
+  {                                                             \
+ _Pragma("GCC diagnostic push")                                 \
+ _Pragma("GCC diagnostic ignored \"-Wtautological-compare\"")   \
+    if (! (cond))                                               \
+    {                                                           \
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,                      \
+                  _ ("Assertion failed at %s:%d. Aborting.\n"), \
+                  __FILE__,                                     \
+                  __LINE__);                                    \
+      GNUNET_abort_ ();                                         \
+    }                                                           \
+ _Pragma("GCC diagnostic pop")                                  \
+  } while (0)
+#else
+/* older GCC/clangs do not support -Wtautological-compare */
 #define GNUNET_assert(cond)                                     \
   do                                                            \
   {                                                             \
@@ -867,7 +905,7 @@ GNUNET_error_type_to_string (enum GNUNET_ErrorType kind);
       GNUNET_abort_ ();                                         \
     }                                                           \
   } while (0)
-
+#endif
 
 /**
  * @ingroup logging
