@@ -1784,6 +1784,13 @@ extern "C" {
  */
 #define GNUNET_MESSAGE_TYPE_SETU_P2P_OVER 572
 
+/**
+ * Signals other peer that all elements are sent.
+ */
+
+#define GNUNET_MESSAGE_TYPE_SETU_P2P_SEND_FULL 710
+
+
 
 /*******************************************************************************
  * SETI message types
@@ -3549,6 +3556,28 @@ extern "C" {
 
 /*********************************************************************************/
 
+/*********************************************************************************/
+/**********************************  Cmd Testing  **********************************/
+/*********************************************************************************/
+
+/**
+ * The initialization message towards gnunet-cmds-helper
+ */
+#define GNUNET_MESSAGE_TYPE_CMDS_HELPER_INIT 1700
+
+/**
+ * The reply message from gnunet-cmds-helper
+ */
+#define GNUNET_MESSAGE_TYPE_CMDS_HELPER_REPLY 1701
+
+#define GNUNET_MESSAGE_TYPE_CMDS_HELPER_PEER_STARTED 1702
+
+#define GNUNET_MESSAGE_TYPE_CMDS_HELPER_ALL_PEERS_STARTED 1703
+
+#define GNUNET_MESSAGE_TYPE_CMDS_HELPER_LOCAL_FINISHED 1704
+
+/*********************************************************************************/
+
 /**
  * Type used to match 'all' message types.
  */
