@@ -1,27 +1,32 @@
 /*
      This file is part of GNUnet.
-     (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
-*/
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+     SPDX-License-Identifier: AGPL3.0-or-later
+ */
 
 /**
- * @file include/gnunet_signal_lib.h
- * @brief functions related to signals
  * @author Christian Grothoff
+ *
+ * @file
+ * Functions related to signals
+ *
+ * @defgroup signal  Signal library
+ * Manage signal handlers.
+ * @{
  */
 
 #ifndef GNUNET_SIGNAL_LIB_H
@@ -41,6 +46,7 @@ extern "C"
  */
 struct GNUNET_SIGNAL_Context;
 
+
 /**
  * A signal handler.  Since different OSes have different signatures
  * for their handlers, the API only gives the most restrictive
@@ -49,7 +55,9 @@ struct GNUNET_SIGNAL_Context;
  * the implementation must guarantee that this handler is not called
  * for signals other than the one that it has been registered for.
  */
-typedef void (*GNUNET_SIGNAL_Handler) (void);
+typedef void
+(*GNUNET_SIGNAL_Handler) (void);
+
 
 /**
  * Install a signal handler that will be run if the
@@ -60,10 +68,12 @@ typedef void (*GNUNET_SIGNAL_Handler) (void);
  * @return context that can be used to restore, NULL on error
  */
 struct GNUNET_SIGNAL_Context *
-GNUNET_SIGNAL_handler_install (int signal, GNUNET_SIGNAL_Handler handler);
+GNUNET_SIGNAL_handler_install (int signal,
+                               GNUNET_SIGNAL_Handler handler);
+
 
 /**
- * Uninstall a previously installed signal hander.
+ * Uninstall a previously installed signal handler.
  *
  * @param ctx context that was returned when the
  *            signal handler was installed
@@ -92,4 +102,7 @@ GNUNET_SIGNAL_raise (const int sig);
 
 /* ifndef GNUNET_SIGNAL_LIB_H */
 #endif
+
+/** @} */  /* end of group */
+
 /* end of gnunet_signal_lib.h */
