@@ -37,13 +37,6 @@
                                                                    syscall, \
                                                                    filename)
 
-/**
- * Hash block of given size.
- *
- * @param block the data to #GNUNET_CRYPTO_hash, length is given as a second argument
- * @param size the length of the data to #GNUNET_CRYPTO_hash in @a block
- * @param ret pointer to where to write the hashcode
- */
 void
 GNUNET_CRYPTO_hash (const void *block,
                     size_t size,
@@ -58,17 +51,6 @@ GNUNET_CRYPTO_hash (const void *block,
 /* ***************** binary-ASCII encoding *************** */
 
 
-/**
- * Convert GNUNET_CRYPTO_hash to ASCII encoding.  The ASCII encoding is rather
- * GNUnet specific.  It was chosen such that it only uses characters
- * in [0-9A-V], can be produced without complex arithmetics and uses a
- * small number of characters.  The GNUnet encoding uses 103
- * characters plus a null terminator.
- *
- * @param block the hash code
- * @param result where to store the encoding (struct GNUNET_CRYPTO_HashAsciiEncoded can be
- *  safely cast to char*, a '\\0' termination is set).
- */
 void
 GNUNET_CRYPTO_hash_to_enc (const struct GNUNET_HashCode *block,
                            struct GNUNET_CRYPTO_HashAsciiEncoded *result)
@@ -86,14 +68,6 @@ GNUNET_CRYPTO_hash_to_enc (const struct GNUNET_HashCode *block,
 }
 
 
-/**
- * Convert ASCII encoding back to hash code.
- *
- * @param enc the encoding
- * @param enclen number of characters in @a enc (without 0-terminator, which can be missing)
- * @param result where to store the hash code
- * @return #GNUNET_OK on success, #GNUNET_SYSERR if result has the wrong encoding
- */
 enum GNUNET_GenericReturnValue
 GNUNET_CRYPTO_hash_from_string2 (const char *enc,
                                  size_t enclen,
@@ -110,19 +84,6 @@ GNUNET_CRYPTO_hash_from_string2 (const char *enc,
 }
 
 
-/**
- * @ingroup hash
- *
- * Compute the distance between 2 hashcodes.  The computation must be
- * fast, not involve bits[0] or bits[4] (they're used elsewhere), and be
- * somewhat consistent. And of course, the result should be a positive
- * number.
- *
- * @param a some hash code
- * @param b some hash code
- * @return a positive number which is a measure for
- *  hashcode proximity.
- */
 unsigned int
 GNUNET_CRYPTO_hash_distance_u32 (const struct GNUNET_HashCode *a,
                                  const struct GNUNET_HashCode *b)
@@ -134,90 +95,53 @@ GNUNET_CRYPTO_hash_distance_u32 (const struct GNUNET_HashCode *a,
 }
 
 
-/**
- * Create a random hash code.
- *
- * @param mode desired quality level
- * @param result hash code that is randomized
- */
 void
 GNUNET_CRYPTO_hash_create_random (enum GNUNET_CRYPTO_Quality mode,
                                   struct GNUNET_HashCode *result)
 {
-  int i;
-
-  for (i = (sizeof(struct GNUNET_HashCode) / sizeof(uint32_t)) - 1; i >= 0; i--)
+  for (ssize_t i = (sizeof(struct GNUNET_HashCode) / sizeof(uint32_t)) - 1;
+       i >= 0;
+       i--)
     result->bits[i] = GNUNET_CRYPTO_random_u32 (mode, UINT32_MAX);
 }
 
 
-/**
- * compute result(delta) = b - a
- *
- * @param a some hash code
- * @param b some hash code
- * @param result set to b - a
- */
 void
 GNUNET_CRYPTO_hash_difference (const struct GNUNET_HashCode *a,
                                const struct GNUNET_HashCode *b,
                                struct GNUNET_HashCode *result)
 {
-  int i;
-
-  for (i = (sizeof(struct GNUNET_HashCode) / sizeof(unsigned int)) - 1; i >= 0;
+  for (ssize_t i = (sizeof(struct GNUNET_HashCode) / sizeof(unsigned int)) - 1;
+       i >= 0;
        i--)
     result->bits[i] = b->bits[i] - a->bits[i];
 }
 
 
-/**
- * compute result(b) = a + delta
- *
- * @param a some hash code
- * @param delta some hash code
- * @param result set to a + delta
- */
 void
 GNUNET_CRYPTO_hash_sum (const struct GNUNET_HashCode *a,
                         const struct GNUNET_HashCode *delta, struct
                         GNUNET_HashCode *result)
 {
-  int i;
-
-  for (i = (sizeof(struct GNUNET_HashCode) / sizeof(unsigned int)) - 1; i >= 0;
+  for (ssize_t i = (sizeof(struct GNUNET_HashCode) / sizeof(unsigned int)) - 1;
+       i >= 0;
        i--)
     result->bits[i] = delta->bits[i] + a->bits[i];
 }
 
 
-/**
- * compute result = a ^ b
- *
- * @param a some hash code
- * @param b some hash code
- * @param result set to a ^ b
- */
 void
 GNUNET_CRYPTO_hash_xor (const struct GNUNET_HashCode *a,
                         const struct GNUNET_HashCode *b,
                         struct GNUNET_HashCode *result)
 {
-  int i;
-
-  for (i = (sizeof(struct GNUNET_HashCode) / sizeof(unsigned int)) - 1; i >= 0;
+  for (ssize_t i = (sizeof(struct GNUNET_HashCode) / sizeof(unsigned int)) - 1;
+       i >= 0;
        i--)
     result->bits[i] = a->bits[i] ^ b->bits[i];
 }
 
 
-/**
- * Convert a hashcode into a key.
- *
- * @param hc hash code that serves to generate the key
- * @param skey set to a valid session key
- * @param iv set to a valid initialization vector
- */
 void
 GNUNET_CRYPTO_hash_to_aes_key (const struct GNUNET_HashCode *hc,
                                struct GNUNET_CRYPTO_SymmetricSessionKey *skey,
@@ -225,29 +149,24 @@ GNUNET_CRYPTO_hash_to_aes_key (const struct GNUNET_HashCode *hc,
                                GNUNET_CRYPTO_SymmetricInitializationVector *iv)
 {
   GNUNET_assert (GNUNET_YES ==
-                 GNUNET_CRYPTO_kdf (skey, sizeof(struct
-                                                 GNUNET_CRYPTO_SymmetricSessionKey),
-                                    "Hash key derivation", strlen (
-                                      "Hash key derivation"),
-                                    hc, sizeof(struct GNUNET_HashCode),
-                                    NULL, 0));
+                 GNUNET_CRYPTO_kdf (
+                   skey,
+                   sizeof(struct GNUNET_CRYPTO_SymmetricSessionKey),
+                   "Hash key derivation",
+                   strlen ("Hash key derivation"),
+                   hc, sizeof(struct GNUNET_HashCode),
+                   NULL, 0));
   GNUNET_assert (GNUNET_YES ==
-                 GNUNET_CRYPTO_kdf (iv, sizeof(struct
-                                               GNUNET_CRYPTO_SymmetricInitializationVector),
-                                    "Initialization vector derivation", strlen (
-                                      "Initialization vector derivation"),
-                                    hc, sizeof(struct GNUNET_HashCode),
-                                    NULL, 0));
+                 GNUNET_CRYPTO_kdf (
+                   iv,
+                   sizeof(struct GNUNET_CRYPTO_SymmetricInitializationVector),
+                   "Initialization vector derivation",
+                   strlen ("Initialization vector derivation"),
+                   hc, sizeof(struct GNUNET_HashCode),
+                   NULL, 0));
 }
 
 
-/**
- * Obtain a bit from a hashcode.
- * @param code the GNUNET_CRYPTO_hash to index bit-wise
- * @param bit index into the hashcode, [0...511] where 0 is the leftmost bit
- *        (bytes in code interpreted big endian)
- * @return Bit \a bit from hashcode \a code, -1 for invalid index
- */
 int
 GNUNET_CRYPTO_hash_get_bit_ltr (const struct GNUNET_HashCode *code,
                                 unsigned int bit)
@@ -257,13 +176,6 @@ GNUNET_CRYPTO_hash_get_bit_ltr (const struct GNUNET_HashCode *code,
 }
 
 
-/**
- * Obtain a bit from a hashcode.
- * @param code the GNUNET_CRYPTO_hash to index bit-wise
- * @param bit index into the hashcode, [0...511] where 0 is the rightmost bit
- *        (bytes in code interpreted little endian)
- * @return Bit \a bit from hashcode \a code, -1 for invalid index
- */
 int
 GNUNET_CRYPTO_hash_get_bit_rtl (const struct GNUNET_HashCode *code,
                                 unsigned int bit)
@@ -273,25 +185,11 @@ GNUNET_CRYPTO_hash_get_bit_rtl (const struct GNUNET_HashCode *code,
 }
 
 
-/**
- * Determine how many low order bits match in two
- * `struct GNUNET_HashCode`s.  i.e. - 010011 and 011111 share
- * the first two lowest order bits, and therefore the
- * return value is two (NOT XOR distance, nor how many
- * bits match absolutely!).
- *
- * @param first the first hashcode
- * @param second the hashcode to compare first to
- *
- * @return the number of bits that match
- */
 unsigned int
 GNUNET_CRYPTO_hash_matching_bits (const struct GNUNET_HashCode *first,
                                   const struct GNUNET_HashCode *second)
 {
-  unsigned int i;
-
-  for (i = 0; i < sizeof(struct GNUNET_HashCode) * 8; i++)
+  for (unsigned int i = 0; i < sizeof(struct GNUNET_HashCode) * 8; i++)
     if (GNUNET_CRYPTO_hash_get_bit_rtl (first, i) !=
         GNUNET_CRYPTO_hash_get_bit_rtl (second, i))
       return i;
@@ -299,25 +197,17 @@ GNUNET_CRYPTO_hash_matching_bits (const struct GNUNET_HashCode *first,
 }
 
 
-/**
- * Compare function for HashCodes, producing a total ordering
- * of all hashcodes.
- *
- * @param h1 some hash code
- * @param h2 some hash code
- * @return 1 if h1 > h2, -1 if h1 < h2 and 0 if h1 == h2.
- */
 int
 GNUNET_CRYPTO_hash_cmp (const struct GNUNET_HashCode *h1,
                         const struct GNUNET_HashCode *h2)
 {
   unsigned int *i1;
   unsigned int *i2;
-  int i;
 
   i1 = (unsigned int *) h1;
   i2 = (unsigned int *) h2;
-  for (i = (sizeof(struct GNUNET_HashCode) / sizeof(unsigned int)) - 1; i >= 0;
+  for (ssize_t i = (sizeof(struct GNUNET_HashCode) / sizeof(unsigned int)) - 1;
+       i >= 0;
        i--)
   {
     if (i1[i] > i2[i])
@@ -329,25 +219,16 @@ GNUNET_CRYPTO_hash_cmp (const struct GNUNET_HashCode *h1,
 }
 
 
-/**
- * Find out which of the two `struct GNUNET_HashCode`s is closer to target
- * in the XOR metric (Kademlia).
- *
- * @param h1 some hash code
- * @param h2 some hash code
- * @param target some hash code
- * @return -1 if h1 is closer, 1 if h2 is closer and 0 if h1==h2.
- */
 int
 GNUNET_CRYPTO_hash_xorcmp (const struct GNUNET_HashCode *h1,
                            const struct GNUNET_HashCode *h2,
                            const struct GNUNET_HashCode *target)
 {
-  int i;
   unsigned int d1;
   unsigned int d2;
 
-  for (i = sizeof(struct GNUNET_HashCode) / sizeof(unsigned int) - 1; i >= 0;
+  for (ssize_t i = sizeof(struct GNUNET_HashCode) / sizeof(unsigned int) - 1;
+       i >= 0;
        i--)
   {
     d1 = ((unsigned int *) h1)[i] ^ ((unsigned int *) target)[i];
@@ -361,14 +242,6 @@ GNUNET_CRYPTO_hash_xorcmp (const struct GNUNET_HashCode *h1,
 }
 
 
-/**
- * @brief Derive an authentication key
- * @param key authentication key
- * @param rkey root key
- * @param salt salt
- * @param salt_len size of the @a salt
- * @param ... pair of void * & size_t for context chunks, terminated by NULL
- */
 void
 GNUNET_CRYPTO_hmac_derive_key (struct GNUNET_CRYPTO_AuthKey *key,
                                const struct
@@ -383,14 +256,6 @@ GNUNET_CRYPTO_hmac_derive_key (struct GNUNET_CRYPTO_AuthKey *key,
 }
 
 
-/**
- * @brief Derive an authentication key
- * @param key authentication key
- * @param rkey root key
- * @param salt salt
- * @param salt_len size of the @a salt
- * @param argp pair of void * & size_t for context chunks, terminated by NULL
- */
 void
 GNUNET_CRYPTO_hmac_derive_key_v (struct GNUNET_CRYPTO_AuthKey *key,
                                  const struct
@@ -405,17 +270,6 @@ GNUNET_CRYPTO_hmac_derive_key_v (struct GNUNET_CRYPTO_AuthKey *key,
 }
 
 
-/**
- * Calculate HMAC of a message (RFC 2104)
- * TODO: Shouldn' this be the standard hmac function and
- * the above be renamed?
- *
- * @param key secret key
- * @param key_len secret key length
- * @param plaintext input plaintext
- * @param plaintext_len length of @a plaintext
- * @param hmac where to store the hmac
- */
 void
 GNUNET_CRYPTO_hmac_raw (const void *key, size_t key_len,
                         const void *plaintext, size_t plaintext_len,
@@ -443,14 +297,6 @@ GNUNET_CRYPTO_hmac_raw (const void *key, size_t key_len,
 }
 
 
-/**
- * Calculate HMAC of a message (RFC 2104)
- *
- * @param key secret key
- * @param plaintext input plaintext
- * @param plaintext_len length of @a plaintext
- * @param hmac where to store the hmac
- */
 void
 GNUNET_CRYPTO_hmac (const struct GNUNET_CRYPTO_AuthKey *key,
                     const void *plaintext, size_t plaintext_len,
@@ -462,9 +308,6 @@ GNUNET_CRYPTO_hmac (const struct GNUNET_CRYPTO_AuthKey *key,
 }
 
 
-/**
- * Context for cummulative hashing.
- */
 struct GNUNET_HashContext
 {
   /**
@@ -474,11 +317,6 @@ struct GNUNET_HashContext
 };
 
 
-/**
- * Start incremental hashing operation.
- *
- * @return context for incremental hash computation
- */
 struct GNUNET_HashContext *
 GNUNET_CRYPTO_hash_context_start ()
 {
@@ -498,13 +336,6 @@ GNUNET_CRYPTO_hash_context_start ()
 }
 
 
-/**
- * Add data to be hashed.
- *
- * @param hc cummulative hash context
- * @param buf data to add
- * @param size number of bytes in @a buf
- */
 void
 GNUNET_CRYPTO_hash_context_read (struct GNUNET_HashContext *hc,
                                  const void *buf,
@@ -516,12 +347,19 @@ GNUNET_CRYPTO_hash_context_read (struct GNUNET_HashContext *hc,
 }
 
 
-/**
- * Finish the hash computation.
- *
- * @param hc hash context to use
- * @param r_hash where to write the latest / final hash code
- */
+struct GNUNET_HashContext *
+GNUNET_CRYPTO_hash_context_copy (const struct GNUNET_HashContext *hc)
+{
+  struct GNUNET_HashContext *cp;
+
+  cp = GNUNET_new (struct GNUNET_HashContext);
+  GNUNET_assert (0 ==
+                 gcry_md_copy (&cp->hd,
+                               hc->hd));
+  return cp;
+}
+
+
 void
 GNUNET_CRYPTO_hash_context_finish (struct GNUNET_HashContext *hc,
                                    struct GNUNET_HashCode *r_hash)
@@ -540,11 +378,6 @@ GNUNET_CRYPTO_hash_context_finish (struct GNUNET_HashContext *hc,
 }
 
 
-/**
- * Abort hashing, do not bother calculating final result.
- *
- * @param hc hash context to destroy
- */
 void
 GNUNET_CRYPTO_hash_context_abort (struct GNUNET_HashContext *hc)
 {
